/******************************************************************
 * moshpit
 * 
 * By Shen Mansell (shenmansell@hotmail.com
 * http://shen.mansell.tripod.com
 *
 * This program requires GCC to be compiled. Download it from
 * http://www.devrs.com/gba
 *
 * Version 1.0 - 7 Oct 2001
 *
 ******************************************************************
 */

#include <stdlib.h>

#include "libs/gba.h"
#include "libs/gbalib.h"
#include "moshpit.h"
#include "main.h"
#include "map.h"

#include "images/font8x8.h"

#define sprite_off  ((Sprite) {512,0,170,0})
#define sprite_clean ((Sprite) {0,0,0,0})

u16 * tpal;
u16 * tiles;
u16 * map0;
u16 * map1;
u16 * map2;
u16 * map3;
u16 * sprite_vram;

u16 joy;
u16 last_joy;

s16 bkg_x, bkg_y;

player_t player;

enemy_t enemy[max_enemies];

u8 level_map [32][64];
room_t rooms [9];

void setup_screen (void)
{
  tpal  = REG_PALETTE_BASE;           // background palette pointer
  
  //tiles = REG_TILE_BASE + 0x4000;     // tiles pointer
  tiles = (u16 *)0x6000000;

  sprite_vram = REG_SPRITE_BASE;

  // oops magic numbers -
  // 0x80 represents 1 palette of 256 colours
  // 0x04 character set for this plane is stored at TILEBASE + 0x4000
  // 0x200 the tilemap (screen memory) for this plane is stored at TILEMAPBASE + 0x800
  REG_BG0CNT =  (1 << 11)+(1<<14);                  // tiles @ TILEBASE+0x4000, tilemap @ TILEMAPBASE
  map0 = &REG_TILE_BASE[8192];
 
  REG_DISPCNT = 0 | BG_MODE_0 | OBJ_2D_MAP | OBJ_ENABLE | BG0_ENABLE ;
  REG_DISPSTAT = 0;
}

void setup_player_game (void)
{
  player.x = 1;
  player.y = 1;
  player.gold = 0;
  player.in_room = 10;
  player.hit_points = 13;
  player.strength = 12;
  player.smarts = 12;
  player.speed = 12;
  player.armor = 0;
  player.sprite = sprite_clean;

  sprite_tile (&player.sprite, '@');

}


void setup_sprites (void)
{
  u16 i;

	//make sure all sprites are displayed outside visible range
  for (i = 0; i < 128; i++)
	{
    OAM[i] = sprite_off;
  }

  for (i = 0; i < 256; i++)              // copy colour palettes
  {
    REG_PALETTE_BASE[i] = font8x8_pal[i];
    REG_SPRITE_PAL_BASE[i] = font8x8_pal[i];
  }

  loadchardata(font8x8_tiles,font8x8_size,&REG_TILE_BASE[0]); 

  sprite_vram = (u16 *) 0x6010000;
  loadchardata(font8x8_tiles,font8x8_size,sprite_vram); 
} 

void down_a_level (void)
{
  OAM[0] = sprite_off;

  blank_map();
  create_level ();  
  player.in_room = 10;
  check_in_room (&player);
}

u8 can_move (u8 x, u8 y)
{
  if ((level_map[y][x] == M_floor) || (level_map[y][x] > M_solid_wall))
    return TRUE;
  else
    return FALSE;
}

void move_player (void)
{
  u8 x,y;

  joy = (volatile u16) REG_P1;

  x = player.x;
  y = player.y;

  if ((!(joy & J_LEFT)) && (last_joy & J_LEFT))
  {
    if ( can_move (x-1,y))
    {
      move_actor (&player, x-1, y);
    }
  }
  if ((!(joy & J_RIGHT)) && (last_joy & J_RIGHT))
  {
    if ( can_move (player.x+1, player.y) )
    {
      move_actor (&player, player.x+1, player.y);
    }
  }
  if ((!(joy & J_DOWN)) && (last_joy & J_DOWN))
  {
    if ( can_move (x,y+1) )
    {
      move_actor (&player, player.x, player.y+1);
    }
  }
  if ((!(joy & J_UP)) && (last_joy & J_UP))
  {
    if ( can_move (player.x, player.y-1) )
    {
      move_actor (&player, player.x, player.y-1);
    }
  }
  if ((!(joy & J_B)) && (last_joy & J_B))
  {
    if (level_map[player.y][player.x] == M_stairs_down)
    {
      down_a_level();

    }
  }

  last_joy = joy;
}

void pause (void)
{
  u16 joy;

  joy = (volatile u16) REG_P1;
  joy = -joy;
  while (joy & J_START) 
  {
    vsync ();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
  while (!(joy & J_START))
  {
    vsync ();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
  while (joy & J_START) 
  {
    vsync ();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
}

void display_all_sprites (void)
{
  REG_BG0HOFS = bkg_x;
  REG_BG0VOFS = bkg_y;
  
  OAM[0] = player.sprite;
}

void draw_around_player (void)
{
    draw_tile (player.x-1, player.y-1);
    draw_tile (player.x, player.y-1);
    draw_tile (player.x+1, player.y-1);
    draw_tile (player.x-1, player.y);
    draw_tile (player.x, player.y);
    draw_tile (player.x+1, player.y);
    draw_tile (player.x-1, player.y+1);
    draw_tile (player.x, player.y+1);
    draw_tile (player.x+1, player.y+1);
}

void move_actor (player_t *p, u8 x, u8 y)
{
  p->x = x;
  p->y = y;

  bkg_x = (player.x << 3)-118;
  bkg_y = (player.y << 3)-78;
  if (bkg_x < 0)
    bkg_x = 0;
  if (bkg_x > 272)
    bkg_x = 272;

  if (bkg_y < 0)
    bkg_y = 0;
  if (bkg_y > 94)
    bkg_y = 94;

  sprite_move (&player.sprite, (player.x <<3) - bkg_x, (player.y <<3) - bkg_y);
  check_in_room (p);
}

void AgbMain(void) {
  
  init_gbalib ();

 
  setup_screen ();
  setup_sprites ();
  setup_player_game ();

  down_a_level ();

  while (1==1)
  { 
    move_player ();
    draw_around_player ();
    
    display_all_sprites ();
    vsync();

  }
  
}
