#include <stdlib.h>
#include "libs/gba.h"
#include "moshpit.h"
#include "main.h"
#include "map.h"

extern u8 level_map [32][64];
extern room_t rooms [9];
extern player_t player;
extern u16 * map0;


void draw_room (u8 r);

void empty_level (void)
{
  u8 x,y;
  

  for (y = 0; y < 32; y++)
  for (x = 0; x < 64; x++)
  {
    level_map [y][x] = M_solid_wall;
  }
}

void add_wall (u8 x, u8 y, u8 wall)
{
  if (level_map[y][x] == M_solid_wall)
    level_map[y][x] = wall;
  else if (level_map[y][x] != M_floor)
    level_map[y][x] |= wall;


}

void create_room (u8 x,u8 y,u8 x2,u8 y2)
{
  u8 tx,ty;

  for (tx = x+1; tx < x2 ; tx++)
  for (ty = y+1; ty < y2 ; ty++)
  {
    if (level_map [ty][tx] != M_floor)
      level_map [ty][tx] = M_floor;   
  }

  // add the top and bottom wall
  for (tx = x+1; tx<x2; tx++)
  {
    if (level_map [y][tx] != M_floor)
      add_wall (tx, y, 10);   

    if (level_map [y2][tx] != M_floor)
      add_wall (tx, y2, 10);   
  }

  // add the side walls
  for (ty = y+1; ty<y2; ty++)
  {
    if (level_map [ty][x] != M_floor)
      add_wall (x, ty, 5);   
    if (level_map [ty][x2] != M_floor)
      add_wall (x2, ty, 5);   
  }

  // add the corner walls
  add_wall (x, y, 6);
  add_wall (x, y2, 3);
  add_wall (x2, y, 12);
  add_wall (x2, y2, 9);
}

// create a hall of M_floor
void create_hall (u8 x, u8 y, u8 x2, u8 y2)
{
  u8 tx,ty;

  tx = x;
  ty = y;

  level_map [ty][tx] = M_floor;

  if (tx < x2)
  while (tx < x2)
  {
    level_map [ty][tx] = M_floor;
    
    if (level_map[ty][tx+1] == M_floor)
      return;
    if (level_map[ty-1][tx] == M_floor)
      return;
    if (level_map[ty+1][tx] == M_floor)
      return;

    tx ++;
  }

  if (tx > x2)
  while (tx > x2)
  {
    level_map [ty][tx] = M_floor;

    if (level_map[ty][tx-1] == M_floor)
      return;
    if (level_map[ty-1][tx] == M_floor)
      return;
    if (level_map[ty+1][tx] == M_floor)
      return;

    tx --;

  }

  level_map [ty][tx] = M_floor;

  if (ty < y2)
  while (ty < y2)
  {
    ty ++;

    level_map [ty][tx] = M_floor;

    if (level_map[ty+1][tx] == M_floor)
      return;
    if (level_map[ty][tx-1] == M_floor)
      return;
    if (level_map[ty][tx+1] == M_floor)
      return;


  }

  if (ty > y2)
  while (ty > y2)
  {
    ty --;

    level_map [ty][tx] = M_floor;
    if (level_map[ty-1][tx] == M_floor)
      return;
    if (level_map[ty][tx-1] == M_floor)
      return;
    if (level_map[ty][tx+1] == M_floor)
      return;
  }
}

// blank the whole map (on the screen)
void blank_map (void)
{
  u16 map_cnt;

  for (map_cnt = 0; map_cnt < 2048; map_cnt++)
  {
    map0[map_cnt] = M_solid_wall;
  }
}


// show the whole map
void show_map (void)
{
  u8 x,y;
  u16 map_cnt;

  // load map into tile memory
  map_cnt = 0;

  for (y=0; y<32; y++)
  for (x=0; x<32; x++)
  {
    map0 [map_cnt] = level_map [y][x]; 
    map_cnt++;
  }

  for (y=0; y<32; y++)
  for (x=32; x<64; x++)
  {
    map0 [map_cnt] = level_map [y][x]; 
    map_cnt++;
  }

}

void refactor_wall (u8 x, u8 y)
{
  u8 t;

  if ((level_map[y][x] > M_floor)  && (level_map[y][x] < M_solid_wall))
  {
    t = 0;
    if ((y>0) && (level_map[y-1][x] > M_floor)  && (level_map[y-1][x] < M_solid_wall))
      t=t+1;
    if ((x<63) && (level_map[y][x+1] > M_floor)  && (level_map[y][x+1] < M_solid_wall))
      t=t+2;
    if ((y<31) && (level_map[y+1][x] > M_floor)  && (level_map[y+1][x] < M_solid_wall))
      t=t+4;
    if ((x>0) && (level_map[y][x-1] > M_floor)  && (level_map[y][x-1] < M_solid_wall))
      t=t+8;

    level_map[y][x] = t;
  }
}


u8 good_place_for_room (u8 x, u8 y, u8 x2, u8 y2)
{
  u8 touching = 0;
  u8 tx,ty;

  if (x >0)
    x--;

  if (y > 0)
    y--;
  
  if (x2 < 63)
    x2++;

  if (y2 < 31)
    y2++;
 
  for (tx = x; tx <= x2; tx++)
  for (ty = y; ty <= y2; ty++)
  {
    if ((level_map [ty][tx] > M_floor) && (level_map [ty][tx] < M_solid_wall))
      return FALSE;
    if (level_map [ty][tx] == M_floor)
      touching ++;
    
  }
  
  if (touching < 5)
    return TRUE;
  else
    return FALSE;
}

void create_level (void)
{
  u8 r;
  u8 x,y,x2,y2;
  u8 curx,cury,lastx,lasty;  

  empty_level ();

  lastx = lasty = 0;

  for (r = 0 ; r < 9; r++)
  {
    do 
    {
      x = rand()%(53);
      y = rand()%(21);
    
      x2 = x+((rand()&57344) >> 13) + 3;
      y2 = y+((rand()&57344) >> 13) + 3;

    } while (good_place_for_room (x,y,x2,y2) == FALSE);
  
    curx = (x+x2+1)>>1;
    cury = (y+y2+1)>>1;
    
    if (lastx > 0)
    {
      create_hall (curx,cury,lastx,lasty);
    }

    create_room (x,y,x2,y2);
    rooms[r].x = x;
    rooms[r].y = y;
    rooms[r].x2 = x2;
    rooms[r].y2 = y2;
    rooms[r].lit = TRUE;
    
    lastx = curx;
    lasty = cury;
  }

  for (y = 0; y<32 ; y++)
  for (x = 0; x<64 ; x++)
    refactor_wall (x,y);

  // place player
  do
  {
    x = rand()&63;
    y = rand()&31;
  } while (level_map[y][x] != M_floor);
  move_actor (&player, x, y);

  // place stairs_down
  do
  {
    x = rand()&63;
    y = rand()&31;
  } while (level_map[y][x] != M_floor);

  level_map[y][x] = M_stairs_down;

}

void draw_tile (u8 x, u8 y)
{
  if (x<32)
  {
    map0 [(y<<5) + x] = level_map[y][x];
  }
  else
  {
    map0 [992 + (y<<5) + x] = level_map[y][x];
  }

}

void draw_room (u8 r)
{
  u8 tx,ty;

  for (tx=rooms[r].x; tx <= rooms[r].x2; tx++)
  for (ty=rooms[r].y; ty <= rooms[r].y2; ty++)
    draw_tile(tx,ty);
}
 
void check_in_room (player_t *p)
{
  u8 t,c;
  t=10;
  c = 0;
  while ((t == 10) && (c < 9))
  {
    if (  (p->x >= rooms[c].x)
        &&(p->x <= rooms[c].x2)
        &&(p->y >= rooms[c].y)
        &&(p->y <= rooms[c].y2)
       )
      t = c;

    c++;
  }

  if ((t < 10) && (p->in_room == 10) && (rooms[t].lit == TRUE))
  {
    // put the room on the map
    draw_room (t);
  }

  p->in_room = t;
}

